from collections.abc import MutableMapping
from pathlib import Path
from typing import Iterator, Any, Hashable, Union
import json


class DataContainer(MutableMapping):

    def __init__(self, path: Union[Path, str]):
        self.path = Path(path)
        if not self.path.exists():
            self.path.mkdir(parents=True)
        self._storage = dict()

    def __setitem__(self, key: Hashable, value: Any) -> None:
        location_json = self.path / f'{key}.json'
        with open(location_json, 'w') as f:
            json.dump(value, f)
        self._storage[key] = value

    def __delitem__(self, key: Hashable) -> None:
        del self._storage[key]
        (self.path / f'{key}.json').unlink()

    def __getitem__(self, key: Hashable) -> Any:
        if key in self._storage:
            return self._storage[key]
        locations_json = self.path / f'{key}.json'
        if locations_json.exists():
            with open(locations_json) as f:
                self._storage[key] = json.load(f)
                return self._storage[key]
        raise KeyError

    def __len__(self) -> int:
        return len(self._storage)

    def __iter__(self) -> Iterator[Any]:
        return iter(self._storage)

    def __contains__(self, key):
        if key in self._storage:
            return True
        return (self.path / f'{key}.json').exists()



from pathlib import Path

SCRIPT_URL = {
    'Karlsruhe': 'https://seatfinder.bibliothek.kit.edu/karlsruhe/getdata.php',
    'Geneva': 'https://seatfinder.bibliothek.kit.edu/unige/getdata.php',
    'Kassel': 'https://seatfinder.bibliothek.kit.edu/kassel/getdata.php',
    'St. Gallen': 'https://seatfinder.bibliothek.kit.edu/unisg_v2/getdata.php',
    'Tuebingen': 'https://seatfinder.bibliothek.kit.edu/tuebingen/getdata.php',
}

LOCATIONS = {
    'Karlsruhe': [
        'LSG',
        'LSM',
        'LST',
        'LSN',
        'LSW',
        'LBS',
        'BIB-N',
        'FBC',
        'FBP',
        'LAF',
        'FBA',
        'FBI',
        'FBM',
        'FBW',
        'FBH',
        'FBD',
        'TheaBib',
        'BLB',
        'WIS',
    ],
    'Geneva': [
        'ARVE',
        'MATHS',
        'ISE',
        'TERRE',
        'OBS',
        'BATELLED',
        'BC',
        'PHILOSOPHES',
        'JURA',
        'CMU',
        'MAIL',
    ],
    'Kassel': [
        'UBA0EP',
        'UBA1EP',
        'UBA2EP',
        'UBA3EP',
        'UBB0EP',
        'UBB0GP',
        'UBB1EP',
        'UBB1GP',
        'UBB2EP',
        'UBB2GP',
        'LeoEG',
        'LeoOG',
    ],
    'St. Gallen': [
        'BIB_EG',
        'BIB_OG',
        '09-116',
        '01_212',
        '01_311',
        'theCO',
        'ZIG_TH',
        'ZIG_Caf',
        'FHS_Bib',
        'KBV_HP',
        'ex_Bie',
    ],
    'Tuebingen': [
        'UBH1',
        'UBB2',
        'UBB2HLS',
        'UBA3A',
        'UBA3C',
        'UBA4A',
        'UBA4B',
        'UBA4C',
        'UBA5A',
        'UBA5B',
        'UBA5C',
        'UBA6A',
        'UBA6B',
        'UBA6C',
        'UBWZA',
        'UBWZB',
        'UBNEG',
        'UBCEG',
        'UBCUG',
        'UBLZN',
    ]
}

DATA_DIR = Path.home() / '.seatfinder/data'



from datetime import datetime, timedelta
from typing import Union, TYPE_CHECKING
import requests

from seatfinder.timeseries_container import TimeseriesContainer

if TYPE_CHECKING:
    from seatfinder import Seatfinder


class TimeseriesLoader:

    def __init__(self, value, seatfinder: 'Seatfinder'):
        self._value = value
        self._seatfinder = seatfinder

    def __getitem__(self, item: Union[datetime, str, slice]):
        if type(item) == slice:
            start, stop, step = item.start, item.stop, item.step
            if start is None:
                raise ValueError("Can't slice from beginning, need to pass a startdate")
            if stop is None:
                stop = datetime.now()
        else:
            start = item
            stop = None
            step = None
        if type(start) != datetime:
            start = datetime.strptime(start, '%Y-%m-%d')
        if not stop:
            return TimeseriesContainer(self._request(start))
        if type(stop) != datetime:
            stop = datetime.strptime(stop, '%Y-%m-%d')
        date = start
        result = TimeseriesContainer()
        while date < stop:
            response = self._request(date)
            for k, v in response.items():
                if k not in result:
                    result[k] = v
                else:
                    result[k].extend(v)
            date += timedelta(days=step or 1)
        return result

    def __delitem__(self, key):
        del self._seatfinder._data[f'{self._value}_{key}']

    def _request(self, date: datetime):
        after = date.strftime('%Y-%m-%d')
        date_before = date + timedelta(days=1)
        before = date_before.strftime('%Y-%m-%d')

        key = f'{self._value}_{after}'
        response = self._seatfinder._data.get(key)

        if not response:
            response = requests.get(self._seatfinder._script_url, {
                'values': self._value,
                'location': ','.join(self._seatfinder._locations),
                'after': after,
                'before': before,
            }).json()
            if datetime.strptime(after, '%Y-%m-%d') < datetime.now() - timedelta(days=1):
                # Never save the current day or the future, since the response will still change
                self._seatfinder._data[key] = response
        return response[0][self._value]



from datetime import datetime
from pathlib import Path

from seatfinder.constants import SCRIPT_URL, DATA_DIR, LOCATIONS
import requests

from seatfinder.data_container import DataContainer
from seatfinder.timeseries_loader import TimeseriesLoader


class Seatfinder:

    def __init__(self, organization, data_dir=DATA_DIR):

        if organization not in SCRIPT_URL:
            valid_organizations = ', '.join(SCRIPT_URL.keys())
            raise ValueError(
                f'"{organization}" is not a valid organization. Valid organizations are: {valid_organizations}"'
            )

        self._locations = LOCATIONS[organization]
        self._script_url = SCRIPT_URL[organization]

        if data_dir:
            data_dir = Path(data_dir)
            if not data_dir.exists():
                data_dir.mkdir(parents=True)
            self._data = DataContainer(path=data_dir / organization)
        else:
            self._data = {}

        self.seat_estimate = TimeseriesLoader('seatestimate', self)
        self.manual_count = TimeseriesLoader('manualcount', self)

    @property
    def locations(self):
        response = self._data.get('locations')

        if not response:
            response = requests.get(self._script_url, {
                'location': ','.join(self._locations),
                'values': 'location',
            }).json()
            self._data['locations'] = response

        return {k: v[0] if len(v) == 1 else v for k, v in response[0]['location'].items()}



import os

from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='seatfinder',
    version='1.0.1',
    url='https://github.com/soerface/py_seatfinder',
    description='Easy access to the data of the KIT seatfinder. '
                'This project is not affiliated in any way with the original author of https://www.seatfinder.de/',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    packages=['seatfinder'],
    install_requires=['requests', 'pandas', 'matplotlib'],
    tests_require=['nose', 'coverage'],
    test_suite='nose.collector',
)



import pandas as pd
import itertools
from datetime import datetime


class TimeseriesContainer(dict):

    def to_dataframe(self) -> pd.DataFrame:
        values = [
            {
                'free_seats': x['free_seats'],
                'occupied_seats': x['occupied_seats'],
                'location_name': x['location_name'],
                'timestamp': datetime.strptime(x['timestamp']['date'], '%Y-%m-%d %H:%M:%S.%f')
            } for x in itertools.chain(*self.values())]
        df = pd.DataFrame(values)
        df.index = pd.MultiIndex.from_arrays(df[['location_name', 'timestamp']].values.T, names=['location_name', 'timestamp'])
        df = df.drop(columns=['location_name', 'timestamp'])
        return df
